package getstars;

import java.awt.Graphics;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class Board {

	private int size = 10;
	private List<Entity> entityList = new ArrayList<Entity>();
	private Random random = new Random();
	private int score = 0;
	private boolean tiredMode = false;
	private static String saveDataFileName = "GetStarsSaveData.txt";
	
	public Board() {
		loadWorld();
		generateEntities();
	}
	
	public int getSize() {
		return size;
	}
	
	public int getEntityCount() {
		return entityList.size();
	}
	
	public Entity getEntity(int index) {
		return entityList.get(index);
	}
	
	public void addEntity(Entity entity) {
		entityList.add(entity);
	}
	
	public Player getPlayer() {
		int index = 0;
		while (index < entityList.size()) {
			Entity tempEntity = entityList.get(index);
			if (tempEntity instanceof Player) {
				return (Player)tempEntity;
			}
			index += 1;
		}
		return null;
	}
	
	public int getScore() {
		return score;
	}
	
	public boolean getTiredMode() {
		return tiredMode;
	}
	
	public void setTiredMode(boolean inputTiredMode) {
		tiredMode = inputTiredMode;
	}
	
	public boolean entityHasConflict(Entity entity) {
		int index = 0;
		while (index < entityList.size()) {
			Entity tempEntity = entityList.get(index);
			if (entity.conflictsWith(tempEntity)) {
				return true;
			}
			index += 1;
		}
		return false;
	}
	
	public void generateBarrier() {
		int tempCount = 0;
		while (tempCount < 1000) {
			Pos tempPos = new Pos(random.nextInt(size - 1), random.nextInt(size - 1));
			Barrier tempBarrier = new Barrier(tempPos, this);
			if (!entityHasConflict(tempBarrier)) {
				addEntity(tempBarrier);
				break;
			}
			tempCount += 1;
		}
	}
	
	public void generateStar() {
		int tempCount = 0;
		while (tempCount < 1000) {
			Pos tempPos = new Pos(random.nextInt(size), random.nextInt(size));
			Star tempStar = new Star(tempPos, this);
			if (!entityHasConflict(tempStar)) {
				addEntity(tempStar);
				break;
			}
			tempCount += 1;
		}
	}
	
	public void generateEntities() {
		Player tempPlayer = getPlayer();
		Pos tempPlayerPos;
		if (tempPlayer == null) {
			tempPlayerPos = new Pos(0, 0);
		} else {
			tempPlayerPos = tempPlayer.getPos();
		}
		entityList = new ArrayList<Entity>();
		tempPlayer = new Player(tempPlayerPos, this);
		addEntity(tempPlayer);
		int tempCount = 0;
		while (tempCount < 3) {
			generateBarrier();
			tempCount += 1;
		}
		tempCount = 0;
		while (tempCount < 10) {
			generateStar();
			tempCount += 1;
		}
	}
	
	public Entity getEntityAtPos(Pos pos) {
		int index = 0;
		while (index < entityList.size()) {
			Entity tempEntity = entityList.get(index);
			if (tempEntity.intersectsWith(pos)) {
				return tempEntity;
			}
			index += 1;
		}
		return null;
	}
	
	public void collectStar(Star star) {
		star.setShouldBeRemoved(true);
		score += 1;
	}
	
	public void tick() {
		int index = 0;
		while (index < entityList.size()) {
			Entity tempEntity = entityList.get(index);
			tempEntity.tick();
			index += 1;
		}
		index = entityList.size() - 1;
		while (index >= 0) {
			Entity tempEntity = entityList.get(index);
			if (tempEntity.getShouldBeRemoved()) {
				entityList.remove(index);
			}
			index -= 1;
		}
		boolean shouldGenerate = true;
		index = 0;
		while (index < entityList.size()) {
			Entity tempEntity = entityList.get(index);
			if (tempEntity instanceof Star) {
				shouldGenerate = false;
			}
			index += 1;
		}
		if (shouldGenerate) {
			saveWorld();
			generateEntities();
		}
	}
	
	public void draw(CustomPanel panel, Graphics graphics) {
		int index = 0;
		while (index < entityList.size()) {
			Entity tempEntity = entityList.get(index);
			tempEntity.draw(panel, graphics);
			index += 1;
		}
	}
	
	public void saveWorld()
	{
		try {
			PrintWriter writer = new PrintWriter(saveDataFileName, "UTF-8");
			writer.println(score);
			writer.close();
		} catch (IOException exception) {
			System.out.println("ERROR: Unable to save!");
		}
	}
	
	public void loadWorld()
	{
		File tempFile = new File(saveDataFileName);
		if (tempFile.exists())
		{
			try {
				InputStream tempInputStream;
				BufferedReader tempBufferedReader;
				String tempLine;
				tempInputStream = new FileInputStream(saveDataFileName);
				tempBufferedReader = new BufferedReader(new InputStreamReader(tempInputStream, "UTF-8"));
				tempLine = tempBufferedReader.readLine();
				score = Integer.parseInt(tempLine);
				tempBufferedReader.close();
			} catch (IOException exception) {
				System.out.println("ERROR: Unable to load save file!");
			}
		} else {
			score = 0;
		}
	}
	
}
