package getstars;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JPanel;

public class CustomPanel extends JPanel {
	
	private static Board board;
	private static int tileSize = 32;
	private static int width;
	private static int height;
	private static Color backgroundColor = new Color(0, 0, 68);
	private static Color textColor = new Color(204, 204, 204);
	private static Font textFont = new Font("Verdana", Font.PLAIN, 18);
	private static Image spriteSheet;
	
	public CustomPanel(Board inputBoard) {
		super();
		board = inputBoard;
		width = tileSize * (board.getSize() + 4);
		height = tileSize * (board.getSize() + 4) + 70;
		try {
			spriteSheet = ImageIO.read(getClass().getResource("sprites.png"));
		} catch (IOException e) {
			System.out.println("ERROR: Unable to open sprite sheet.");
		}
	}
	
	public void paint(Graphics graphics) {
		graphics.setColor(backgroundColor);
		graphics.fillRect(0, 0, width, height);
		
		drawWalls(graphics);
		board.draw(this, graphics);
		
		graphics.setFont(textFont);
		graphics.setColor(textColor);
		graphics.drawString("Stars: " + board.getScore(), tileSize, height - 45);
	}
	
	public void drawSprite(Graphics graphics, Pos pos, int which) {
		int tempPosX = (which % 4) * 16;
		int tempPosY = (which / 4) * 16;
		graphics.drawImage(spriteSheet, (pos.getX() + 2) * tileSize, (pos.getY() + 2) * tileSize, (pos.getX() + 2) * tileSize + tileSize, (pos.getY() + 2) * tileSize + tileSize,
				tempPosX, tempPosY, tempPosX + 16, tempPosY + 16, null);
	}
	
	public void drawWalls(Graphics graphics) {
		Pos tempPos = new Pos(-1, -1);
		while (tempPos.getX() <= board.getSize()) {
			tempPos.setY(-1);
			drawSprite(graphics, tempPos, 8);
			tempPos.setY(board.getSize());
			drawSprite(graphics, tempPos, 8);
			tempPos.setX(tempPos.getX() + 1);
		}
		tempPos = new Pos(-1, 0);
		while (tempPos.getY() < board.getSize()) {
			tempPos.setX(-1);
			drawSprite(graphics, tempPos, 8);
			tempPos.setX(board.getSize());
			drawSprite(graphics, tempPos, 8);
			tempPos.setY(tempPos.getY() + 1);
		}
	}
	
	public int getWidth() {
		return width;
	}
	
	public int getHeight() {
		return height;
	}
	
}