package getstars;

import java.awt.Graphics;

public class Player extends Entity {

	private boolean isWalking = false;
	private Pos walkOffset = new Pos(0, 0);
	private int walkDelay = 0;
	
	public Player(Pos pos, Board board) {
		super(pos, 1, board);
	}
	
	@Override
	public void tick() {
		if (isWalking) {
			if (walkDelay < 1) {
				walk();
				walkDelay = 5;
			}
		}
		if (walkDelay > 0) {
			walkDelay -= 1;
		}
	}

	@Override
	public void draw(CustomPanel panel, Graphics graphics) {
		panel.drawSprite(graphics, getPos(), 0);
	}

	public void startWalking(Pos offset) {
		walkOffset = offset;
		isWalking = true;
	}
	
	public void stopWalking(Pos offset) {
		if (offset.getX() < 0 && walkOffset.getX() < 0) {
			walkOffset.setX(0);
			isWalking = false;
		}
		if (offset.getX() > 0 && walkOffset.getX() > 0) {
			walkOffset.setX(0);
			isWalking = false;
		}
		if (offset.getY() < 0 && walkOffset.getY() < 0) {
			walkOffset.setY(0);
			isWalking = false;
		}
		if (offset.getY() > 0 && walkOffset.getY() > 0) {
			walkOffset.setY(0);
			isWalking = false;
		}
	}
	
	public void walk() {
		Pos tempPos = getPos().copy();
		tempPos.add(walkOffset);
		if (tempPos.getX() < 0 || tempPos.getX() >= getBoard().getSize() || tempPos.getY() < 0 || tempPos.getY() >= getBoard().getSize()) {
			return;
		}
		Entity tempEntity = getBoard().getEntityAtPos(tempPos);
		if (tempEntity != null) {
			if (tempEntity instanceof Star) {
				getBoard().collectStar((Star)tempEntity);
			}
			if (tempEntity instanceof Barrier) {
				return;
			}
		}
		getPos().set(tempPos);
	}
	
}
